﻿using System.Runtime.InteropServices;
using UnityEngine;
using Windows.Kinect;

public class CoordinateMapperManager : MonoBehaviour
{
    private DepthSpacePoint[] _DepthCoordinates;

    private KinectModule.SourceBuffer _SourceBuffer;
    private KinectModule.KinectAccessManager _KinectAccessManager;

    long frameCount = 0;

    double elapsedCounter = 0.0;
    double fps = 0.0;
    
    Texture2D _ColorRGBX;

    public Texture2D ColorTexture
    { get { return _ColorRGBX; } }
    public byte[] BodyIndexBuffer
    { get { return _SourceBuffer.BodyIndexBuffer; } }
    public DepthSpacePoint[] DepthCoordinates
    { get { return _DepthCoordinates; } }
    public KinectModule.IBody[] BodyBuffer
    { get { return _SourceBuffer.BodyBuffer; } }

    void Awake()
    {
        _ColorRGBX = new Texture2D (KinectModule.KinectConstants.ColorWidth,
                                    KinectModule.KinectConstants.ColorHeight,
                                    TextureFormat.RGBA32, false);

        _DepthCoordinates = new DepthSpacePoint[KinectModule.KinectConstants.ColorWidth *
                                                KinectModule.KinectConstants.ColorHeight];

        _SourceBuffer = new KinectModule.SourceBuffer();

        _KinectAccessManager = new KinectModule.KinectAccessManager();
        _KinectAccessManager.Dispose();
    }

    void OnEnable()
    {
        _KinectAccessManager.Access();
    }

    void OnDisable()
    {
        _KinectAccessManager.Dispose();
    }

    void OnGUI () 
    {
        GUI.Box (new Rect(10, 10, 200, 30), "FPS: " + fps.ToString("0.00"));

        /*if (_KinectAccessManager.NullFrame)
            GUI.Box (new Rect(10, 50, 200, 30), "NULL MSFR Frame");*/
    }

    void MapColorToDepth()
    {
        var DepthData = GCHandle.Alloc(_SourceBuffer.DepthBuffer, GCHandleType.Pinned);
        var DepthCoordinatesData = GCHandle.Alloc(_DepthCoordinates, GCHandleType.Pinned);

        _KinectAccessManager.CoordinateMapper.MapColorFrameToDepthSpaceUsingIntPtr(
            DepthData.AddrOfPinnedObject(),
            (uint)_SourceBuffer.DepthBuffer.Length * sizeof(ushort),
            DepthCoordinatesData.AddrOfPinnedObject(),
            (uint)_DepthCoordinates.Length);

        DepthCoordinatesData.Free();
        DepthData.Free();
    }

    void UpdateTexture()
    {
        _ColorRGBX.LoadRawTextureData(_SourceBuffer.ColorBuffer);
        _ColorRGBX.Apply();
    }

    void GetFPS()
    {
        elapsedCounter += Time.deltaTime;
        if (elapsedCounter > 1.0)
        {
            fps = frameCount / elapsedCounter;
            frameCount = 0;
            elapsedCounter = 0.0;
        }
        frameCount++;
    }
    
    void Update()
    {
        GetFPS();
        
        var sourceFrame = new KinectModule.RealMultiSourceFrame(_KinectAccessManager.SourceFrame);
        
        if (!_KinectAccessManager.NullFrame)
        {
            _SourceBuffer.UpdateBuffers(sourceFrame);

            MapColorToDepth();

            UpdateTexture();
        }
    }

    void OnApplicationQuit()
    {
        _KinectAccessManager.Dispose();
    }
}
