﻿using System;
using UnityEngine;
using Windows.Kinect;

public class CoordinateMapperView : MonoBehaviour
{
    private CoordinateMapperManager manager;

	private ComputeBuffer DepthBuffer,
                          BodyIndexBuffer;

	private DepthSpacePoint[] DepthPoints;
	private byte[] BodyIndexPoints;

    void Start ()
	{
        manager = GameObject.Find("InGameManagers").GetComponent<CoordinateMapperManager>();

        var material = GetComponent<Renderer>().material;

        material.SetTexture("_MainTex", manager.ColorTexture);

        DepthBuffer     = GenerateBuffer("depthCoordinates", sizeof(float) * 2,
                                         manager.DepthCoordinates, material);
        BodyIndexBuffer = GenerateBuffer("bodyIndexBuffer", sizeof(float),
                                         manager.BodyIndexBuffer, material);
    }

    ComputeBuffer GenerateBuffer(string name, int stride, Array points, Material material)
    {
        var buffer = new ComputeBuffer(points.Length, stride);
        material.SetBuffer(name, buffer);
        return buffer;
    }

	void Update()
	{
		//TODO: fix perf on this call.
		DepthBuffer.SetData(manager.DepthCoordinates);
        
		BodyIndexBuffer.SetData(Array.ConvertAll(manager.BodyIndexBuffer, Convert.ToSingle));
	}
	
	void OnDisable() 
	{
        DepthBuffer.Release();   
        BodyIndexBuffer.Release();
    }
}
