﻿using JudgeModule;
using System.Collections.Generic;
using System.Linq;
using UnityEngine;

public class InGameManager : MonoBehaviour
{    
    private float             baseTime,
                              delay = 2000,
                              delayBase;
    private bool              IsInitTime,
                              IsDelayed,
                              onResult = false;

    private GameObject        offset,
                              noteobj,
                              deactives,
                              appear,
                              disappear,
                              judgetext,
                              motionSampleDisplayPrefab;
    
    private InputManager      input;
    private NoteManager       notemanager;
    private JudgeManager      judgemanager;

    private void MakeControllers()
    {
        foreach (var name in new string[]
        { "SBT", "LBT", "SMO", "LMO", "BeatLine", "MeasureLine" })
        {
            Enumerable.Range(0, 20).ToList().ForEach(x =>
            {
                var obj = Instantiate(Resources.Load(name, typeof(GameObject)),
                    appear.transform) as GameObject;
                obj.name = name;

                var controller = obj.AddComponent<Controller>();
                controller.Instance = new Note(0, 0);

                if (obj.name != "BeatLine" &&
                    obj.name != "MeasureLine")
                    obj.transform.localScale = new Vector3(1, 1.0f / 3, 1);

                obj.gameObject.SetActive(false);
            });
        }
    }

    private void SetManagers(Dictionary<string, GameObject> objects)
    {
        notemanager = new NoteManager(
            objects,
            GameManager.Instance.CurrentTrack.BPM,
            GameManager.Instance.CurrentTrack.LastNote);
        judgemanager = new JudgeManager(
            input,
            ShowResult,
            objects,
            GameManager.Instance.CurrentTrack.BPM);
    }

    private void SetObjectRef()
    {
        offset    = GameObject.Find("Offset");
        noteobj   = GameObject.Find("Noteobj");
        deactives = GameObject.Find("Deactives");
        appear    = GameObject.Find("AppearNote");
        disappear = GameObject.Find("DisappearNote");
        judgetext = GameObject.Find("Judge");
        input     = GetComponent<InputManager>();

        motionSampleDisplayPrefab = Resources.Load("Motion Sample Display")
                                    as GameObject;

        MakeControllers();

        SetManagers(new Dictionary<string, GameObject>
        {
            { "offset",    offset },
            { "noteobj",   noteobj },
            { "deactives", deactives },
            { "appear",    appear },
            { "disappear", disappear },
            { "judgetext", judgetext },
            { "sample",    motionSampleDisplayPrefab },
            { "guage",     GameObject.Find("Motion Guage") }
        });
    }

    private void JudgeInit()
    {
        judgetext.SetActive(false);
        baseTime = 0f;
        IsInitTime = false;
        delayBase = (appear.transform.position.x - offset.transform.position.x)
            * GameManager.Instance.CurrentTrack.BPM / (60 * 1000);
    }

    private void SetSound()
    {
        var sounds = GameObject.Find("Sounds");
        sounds.transform.Cast<Transform>()
                        .ToList()
                        .ForEach(x => x.gameObject.SetActive(false));
        if (GameManager.Instance.CurrentTrack.BGM != null)
            sounds.transform.Find(GameManager.Instance.CurrentTrack.BGM)
                  .gameObject.SetActive(true);
    }
    
    private void Start()
    {
        SetObjectRef();
        JudgeInit();
        SetSound();
    }
    
    private void Update()
    {
        float current = Time.time * 1000;
        if (!IsInitTime)
        {
            baseTime = current;
            IsInitTime = true;
        }

        float timing = current - baseTime - delayBase - delay;
                
        notemanager.Controll(timing);

        if (notemanager.IsEnd)
        {
            ShowResult();
            return;
        }

        JudgeNote(notemanager.CurrentBT, timing);
        JudgeNote(notemanager.CurrentSMO, timing);
        JudgeNote(notemanager.CurrentLMO, timing);
    }

    private void JudgeNote(Note note, float timing)
    {
        if (note != null)
            judgemanager.JudgeNote(note, timing);
    }

    private void ShowResult()
    {
        if (!onResult)
        {
            onResult = true;

            Destroy(GetComponent<InputManager>());
            Destroy(GetComponent<MotionView>());
            Destroy(GetComponent<CoordinateMapperManager>());
            Destroy(this);

            GameManager.Instance.SceneTransition("Result");
        }
    }
}
