﻿using MotionAnalysis;
using StatusConvert;
using System.Linq;
using UnityEngine;

public class InputManager : MonoBehaviour
{
    private bool        IsButtonDownPrev
    { get; set; }
    public bool         IsButtonDown
    { private get; set; }
    
    private MotionState PrevMotionState
    { get; set; }
    public MotionState  CurrentMotionState
    { private get; set; }

    public InputStatus  ShortButtonStat
    {
        get
        {
            if (!IsButtonDownPrev &&
                IsButtonDown)
                return InputStatus.Entered;
            return InputStatus.None;
        }
    }

    public InputStatus  LongButtonStat
    {
        get
        {
            if (!IsButtonDownPrev &&
                IsButtonDown)
                return InputStatus.Entered;
            if (IsButtonDownPrev &&
                IsButtonDown)
                return InputStatus.Continuing;
            if (IsButtonDownPrev &&
                !IsButtonDown)
                return InputStatus.Stopped;
            return InputStatus.None;
        }
    }

    public InputStatus  ShortMotionStat
    {
        get
        {
            if ((new string[]
            {
                "Clap",
                "Jump",
                "PushUpLeft",
                "PushUpRight",
                "PushUpBoth",
                "JumpPushUpLeft",
                "JumpPushUpRight",
                "JumpPushUpBoth",
                "GuardLeft",
                "GuardRight",
                "GuardBoth"
            }).Select(x => MotionToInput(x, "short"))
              .Where(x => x == InputStatus.Entered)
              .Count() == 0)
                return InputStatus.None;
            return InputStatus.Entered;
        }
    }

    public InputStatus  LongMotionStat
    {
        get
        {
            if ((new string[]
            {
                "HandUpLeft",
                "HandUpRight",
                "HandUpBoth",
                "HandDownLeft",
                "HandDownRight",
                "HandDownBoth",
                "Jesus",
                "HeadphoneLeft",
                "HeadphoneRight",
                "HeadphoneBoth",
                "OnTheTable"
            }).Select(x => MotionToInput(x, "long"))
              .Where(x => x == InputStatus.Entered ||
                          x == InputStatus.Continuing)
              .Count() == 0)
                return InputStatus.None;
            return InputStatus.Entered;
        }
    }

    private void Update()
    {
        IsButtonDownPrev = IsButtonDown;
        PrevMotionState = CurrentMotionState;
    }

    public InputStatus MotionToInput(string name, string type)
    {
        if (name.Contains("Both"))
        {
            var tmpname = name.Substring(0, name.IndexOf("Both"));

            var left  = MotionToInput(tmpname + "Left",  type);
            var right = MotionToInput(tmpname + "Right", type);

            return left == right ? left : InputStatus.None;
        }

        MotionStatus prev = MotionStatusExtractor.Extract(name, PrevMotionState),
                     curr = MotionStatusExtractor.Extract(name, CurrentMotionState);

        return type.Equals("long") ? LongMotionStatus(prev, curr) : ShortMotionStatus(prev, curr);
    }

    private InputStatus LongMotionStatus(MotionStatus prev, MotionStatus curr)
    {
        if (prev == MotionStatus.None &&
            (curr & MotionStatus.Prepared) != MotionStatus.None)
            return InputStatus.Entered;
        if ((prev & MotionStatus.Done) != MotionStatus.None &&
            curr == MotionStatus.None)
            return InputStatus.Stopped;
        if (prev == curr &&
            (prev & (MotionStatus.Prepared | MotionStatus.Done)) != MotionStatus.None)
            return InputStatus.Continuing;

        return InputStatus.None;
    }

    private InputStatus ShortMotionStatus(MotionStatus prev, MotionStatus curr)
    {
        if (prev == MotionStatus.Prepared &&
            curr == MotionStatus.Done)
            return InputStatus.Entered;

        return InputStatus.None;
    }
}
