﻿using StatusConvert;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;

public class MotionTextManager : MonoBehaviour
{
    private InputManager input;
    private Text         textShort,
                         textLong;
    private readonly string[] listShort = new string[]
                              {
                                  "Clap",
                                  "Jump",
                                  "PushUpLeft",
                                  "PushUpRight",
                                  "PushUpBoth",
                                  "JumpPushUpLeft",
                                  "JumpPushUpRight",
                                  "JumpPushUpBoth",
                                  "GuardLeft",
                                  "GuardRight",
                                  "GuardBoth"
                              },
                              listLong = new string[]
                              {
                                  "HandUpLeft",
                                  "HandUpRight",
                                  "HandUpBoth",
                                  "HandDownLeft",
                                  "HandDownRight",
                                  "HandDownBoth",
                                  "Jesus",
                                  "HeadphoneLeft",
                                  "HeadphoneRight",
                                  "HeadphoneBoth",
                                  "OnTheTable"
                              },
                              printShort = new string[]
                              {
                                  "Clap:                   ",
                                  "Jump:                  ",
                                  "PushUpLeft:          ",
                                  "PushUpRight:        ",
                                  "PushUpBoth:         ",
                                  "JumpPushUpLeft:   ",
                                  "JumpPushUpRight: ",
                                  "JumpPushUpBoth:  ",
                                  "GuardLeft:             ",
                                  "GuardRight:           ",
                                  "GuardBoth:            "
                              },
                              printLong = new string[]
                              {
                                  "HandUpLeft:       ",
                                  "HandUpRight:     ",
                                  "HandUpBoth:      ",
                                  "HandDownLeft:   ",
                                  "HandDownRight: ",
                                  "HandDownBoth:  ",
                                  "Jesus:                ",
                                  "HeadphoneLeft:   ",
                                  "HeadphoneRight: ",
                                  "HeadphoneBoth:  ",
                                  "OnTheTable:       "
                              };

    private Dictionary<string, Dictionary<InputStatus, bool>> check =
        new Dictionary<string, Dictionary<InputStatus, bool>>();

    private void Start()
    {
        textShort = GameObject.Find("ShortMotion").GetComponent<Text>();
        textLong  = GameObject.Find("LongMotion") .GetComponent<Text>();

        input      = GameObject.Find("InGameManagers").GetComponent<InputManager>();

        foreach (var x in listShort)
            check[x] = new Dictionary<InputStatus, bool>
            {
                { InputStatus.Entered,    false },
                { InputStatus.Continuing, false },
                { InputStatus.Stopped,    false }
            };

        foreach (var x in listLong)
            check[x] = new Dictionary<InputStatus, bool>
            {
                { InputStatus.Entered,    false },
                { InputStatus.Continuing, false },
                { InputStatus.Stopped,    false }
            };
    }

    // Update is called once per frame
    private void Update()
    {
        UpdateText(textShort, listShort, printShort, "short");
        UpdateText(textLong,  listLong,  printLong,  "long");

    }

    private void UpdateText(Text text, string[] list, string[] print, string type)
    {
        text.text = "";
        for (int i = 0; i < list.Length; ++i)
        {
            var status = input.MotionToInput(list[i], type);
            if (status != InputStatus.None)
                check[list[i]][status] = true;
            text.text += (print[i] + checktostr(check[list[i]]) + "\n");
        }
    }

    private string checktostr(Dictionary<InputStatus, bool> chk)
    {
        string str = "";
        str += (chk[InputStatus.Entered]    ? "E" : " ");
        str += (chk[InputStatus.Continuing] ? "C" : " ");
        str += (chk[InputStatus.Stopped]    ? "S" : " ");

        return str;
    }
}
