﻿using MotionAnalysis;
using System.Collections.Generic;
using System.Linq;
using UnityEngine;

public class MotionView : MonoBehaviour
{
    private CoordinateMapperManager         Manager;
    private InputManager                    Input;
    private List<KinectModule.IBody>        Body;
    private MotionDiscriminator             Discriminator = new MotionDiscriminator();

    // Use this for initialization
    void Start ()
    {
        var ingameManager = GameObject.Find("InGameManagers");
        Manager = ingameManager.GetComponent<CoordinateMapperManager>();
        Input   = ingameManager.GetComponent<InputManager>();
    }
    
    // Update is called once per frame
    void Update ()
    {
        if (InitDiscriminator())
            Input.CurrentMotionState = DetermineState();
    }

    bool InitDiscriminator()
    {
        if (Discriminator.IsPreseted)
            return true;

        Body = Manager.BodyBuffer.ToList();

        var body0 = Body[0];
        if (body0 == null)
            return false;

        Discriminator.Preset(body0);
        if (!Discriminator.IsPreseted)
            return false;

        return true;
    }

    MotionState DetermineState()
    {
        Body = Manager.BodyBuffer.ToList();
        var tracked = Body.Where(x => x != null &&
                                      x.IsTracked)
                          .ToList();
        if (tracked.Count()   == 0 ||
            tracked[0].Joints == null)
            return MotionState.UNKNOWN;

        Discriminator.Update(tracked[0]);

        return Discriminator.Motion;
    }
}
